@@ -49,7 +49,7 @@ class File(object):
         if isinstance(filename, File):
             return
         elif not is_text(filename):
-            Log.error('Expecting str, not {{type}}', type=type(filename).__name__)
+            Log.error("Expecting str, not {{type}}", type=type(filename).__name__)
 
         self.key = base642bytearray(key)
         self._mime_type = mime_type
@@ -83,15 +83,15 @@ class File(object):
 
     @property
     def timestamp(self):
-        output = os.path.getmtime(self.abspath)
+        output = os.path.getmtime(self.abs_path)
         return output
 
     @property
-    def filename(self):
+    def rel_path(self):
         return self._filename
 
     @property
-    def abspath(self):
+    def abs_path(self):
         if self._filename.startswith("~"):
             home_path = os.path.expanduser("~")
             if os.sep == "\\":
@@ -106,6 +106,15 @@ class File(object):
             else:
                 return os.path.abspath(self._filename)
 
+    @property
+    def os_path(self):
+        """
+        :return: OS-specific path
+        """
+        if os.sep == "/":
+            return self.abs_path
+        return str(self.abs_path).lstrip("/")
+
     def add_suffix(self, suffix):
         """
         ADD suffix TO THE filename (NOT INCLUDING THE FILE EXTENSION)
@@ -121,8 +130,8 @@ class File(object):
             return parts[-1]
 
     @property
-    def name(self):
-        parts = self.abspath.split("/")[-1].split(".")
+    def stem(self):
+        parts = self.abs_path.split("/")[-1].split(".")
         if len(parts) == 1:
             return parts[0]
         else:
@@ -131,15 +140,15 @@ class File(object):
     @property
     def mime_type(self):
         if not self._mime_type:
-            if self.abspath.endswith(".js"):
+            if self.abs_path.endswith(".js"):
                 self._mime_type = "application/javascript"
-            elif self.abspath.endswith(".css"):
+            elif self.abs_path.endswith(".css"):
                 self._mime_type = "text/css"
-            elif self.abspath.endswith(".json"):
+            elif self.abs_path.endswith(".json"):
                 self._mime_type = mimetype.JSON
             else:
                 mime = MimeTypes()
-                self._mime_type, _ = mime.guess_type(self.abspath)
+                self._mime_type, _ = mime.guess_type(self.abs_path)
                 if not self._mime_type:
                     self._mime_type = mimetype.BINARY
         return self._mime_type
@@ -212,14 +221,15 @@ class File(object):
                 content = f.read().decode(encoding)
                 return content
 
-    def read_zipfile(self, encoding='utf8'):
+    def read_zipfile(self, encoding="utf8"):
         """
         READ FIRST FILE IN ZIP FILE
         :param encoding:
         :return: STRING
         """
         from zipfile import ZipFile
-        with ZipFile(self.abspath) as zipped:
+
+        with ZipFile(self.abs_path) as zipped:
             for num, zip_name in enumerate(zipped.namelist()):
                 return zipped.open(zip_name).read().decode(encoding)
 
@@ -233,7 +243,7 @@ class File(object):
 
         content = self.read(encoding=encoding)
         value = json2value(content, flexible=flexible)
-        abspath = self.abspath
+        abspath = self.abs_path
         if os.sep == "\\":
             abspath = "/" + abspath.replace(os.sep, "/")
         return get_module("mo_json_config").expand(value, "file://" + abspath)
@@ -251,7 +261,9 @@ class File(object):
                 else:
                     return f.read()
         except Exception as e:
-            Log.error(u"Problem reading file {{filename}}", filename=self.abspath, cause=e)
+            Log.error(
+                "Problem reading file {{filename}}", filename=self.abs_path, cause=e
+            )
 
     def write_bytes(self, content):
         if not self.parent.exists:
@@ -271,7 +283,10 @@ class File(object):
             self.parent.create()
         with open(self._filename, "wb") as f:
             if is_list(content) and self.key:
-                Log.error(u"list of data and keys are not supported, encrypt before sending to file")
+                Log.error(
+                    "list of data and keys are not supported, encrypt before sending to"
+                    " file"
+                )
 
             if is_list(content):
                 pass
@@ -282,9 +297,10 @@ class File(object):
 
             for d in content:
                 if not is_text(d):
-                    Log.error(u"Expecting unicode data only")
+                    Log.error("Expecting unicode data only")
                 if self.key:
                     from mo_math.aes_crypto import encrypt
+
                     f.write(encrypt(d, self.key).encode("utf8"))
                 else:
                     f.write(d.encode("utf8"))
@@ -302,13 +318,17 @@ class File(object):
 
                 with io.open(path, "rb") as f:
                     for line in f:
-                        yield line.decode('utf8').rstrip()
+                        yield line.decode("utf8").rstrip()
             except Exception as e:
-                Log.error(u"Can not read line from {{filename}}", filename=self._filename, cause=e)
+                Log.error(
+                    "Can not read line from {{filename}}",
+                    filename=self._filename,
+                    cause=e,
+                )
 
         return output()
 
-    def append(self, content, encoding='utf8'):
+    def append(self, content, encoding="utf8"):
         """
         add a line to file
         """
@@ -316,12 +336,12 @@ class File(object):
             self.parent.create()
         with open(self._filename, "ab") as output_file:
             if not is_text(content):
-                Log.error(u"expecting to write unicode only")
+                Log.error("expecting to write unicode only")
             output_file.write(content.encode(encoding))
             output_file.write(b"\n")
 
     def __len__(self):
-        return os.path.getsize(self.abspath)
+        return os.path.getsize(self.abs_path)
 
     def add(self, content):
         return self.append(content)
@@ -333,12 +353,12 @@ class File(object):
             with open(self._filename, "ab") as output_file:
                 for c in content:
                     if not isinstance(c, text):
-                        Log.error(u"expecting to write unicode only")
+                        Log.error("expecting to write unicode only")
 
                     output_file.write(c.encode("utf8"))
                     output_file.write(b"\n")
         except Exception as e:
-            Log.error(u"Could not write to file", e)
+            Log.error("Could not write to file", e)
 
     def delete(self):
         try:
@@ -349,35 +369,47 @@ class File(object):
             return self
         except Exception as e:
             e = Except.wrap(e)
-            if u"The system cannot find the path specified" in e:
+            if "The system cannot find the path specified" in e:
                 return
-            Log.error(u"Could not remove file", e)
+            Log.error("Could not remove file", e)
 
     def backup(self):
         path = self._filename.split("/")
         names = path[-1].split(".")
-        if len(names) == 1 or names[0] == '':
-            backup = File(self._filename + ".backup " + datetime.utcnow().strftime("%Y%m%d %H%M%S"))
+        if len(names) == 1 or names[0] == "":
+            backup = File(
+                self._filename
+                + ".backup "
+                + datetime.utcnow().strftime("%Y%m%d %H%M%S")
+            )
         else:
             backup = File.new_instance(
                 "/".join(path[:-1]),
-                ".".join(names[:-1]) + ".backup " + datetime.now().strftime("%Y%m%d %H%M%S") + "." + names[-1]
+                ".".join(names[:-1])
+                + ".backup "
+                + datetime.now().strftime("%Y%m%d %H%M%S")
+                + "."
+                + names[-1],
             )
         File.copy(self, backup)
         return backup
 
     def create(self):
         try:
-            os.makedirs(os_path(self.abspath))
+            os.makedirs(self.os_path)
         except FileExistsError:
             pass
         except Exception as e:
-            Log.error(u"Could not make directory {{dir_name}}", dir_name=self._filename, cause=e)
+            Log.error(
+                "Could not make directory {{dir_name}}",
+                dir_name=self._filename,
+                cause=e,
+            )
 
     @property
     def children(self):
         try:
-            return [File(self._filename + "/" + c) for c in os.listdir(self.filename)]
+            return [File(self._filename + "/" + c) for c in os.listdir(self.rel_path)]
         except FileNotFoundError:
             return []
 
@@ -385,14 +417,14 @@ class File(object):
     def decendants(self):
         yield self
         if self.is_directory():
-            for c in os.listdir(os_path(self.abspath)):
+            for c in os.listdir(self.os_path):
                 child = File(self._filename + "/" + c)
                 for cc in child.decendants:
                     yield cc
 
     @property
     def leaves(self):
-        for c in os.listdir(os_path(self.abspath)):
+        for c in os.listdir(self.os_path):
             child = File(self._filename + "/" + c)
             if child.is_directory():
                 for l in child.leaves:
@@ -415,7 +447,7 @@ class File(object):
             return True
         try:
             return os.path.exists(self._filename)
-        except Exception as e:
+        except Exception:
             return False
 
     @property
@@ -446,16 +478,16 @@ class File(object):
         return self._filename
 
     def __unicode__(self):
-        return self.abspath
+        return self.abs_path
 
     def __eq__(self, other):
-        return isinstance(other, File) and other.abspath == self.abspath
+        return isinstance(other, File) and other.abs_path == self.abs_path
 
     def __hash__(self):
-        return self.abspath.__hash__()
+        return self.abs_path.__hash__()
 
     def __str__(self):
-        return self.abspath
+        return self.abs_path
 
 
 class TempDirectory(File):
@@ -476,7 +508,12 @@ class TempDirectory(File):
     def __exit__(self, exc_type, exc_val, exc_tb):
         from mo_threads import Thread
 
-        Thread.run("delete dir " + self.name, delete_daemon, file=self, caller_stack=get_stacktrace(1)).release()
+        Thread.run(
+            "delete dir " + self.stem,
+            delete_daemon,
+            file=self,
+            caller_stack=get_stacktrace(1),
+        ).release()
 
 
 class TempFile(File):
@@ -501,12 +538,17 @@ class TempFile(File):
     def __exit__(self, exc_type, exc_val, exc_tb):
         from mo_threads import Thread
 
-        Thread.run("delete file " + self.name, delete_daemon, file=self, caller_stack=get_stacktrace(1)).release()
+        Thread.run(
+            "delete file " + self.rel_path,
+            delete_daemon,
+            file=self,
+            caller_stack=get_stacktrace(1),
+        ).release()
 
 
 def _copy(from_, to_):
     if from_.is_directory():
-        for c in os.listdir(os_path(from_.abspath)):
+        for c in os.listdir(from_.os_path):
             _copy(from_ / c, to_ / c)
     else:
         File.new_instance(to_).write_bytes(File.new_instance(from_).read_bytes())
@@ -523,27 +565,32 @@ def datetime2string(value, format="%Y-%m-%d %H:%M:%S"):
     try:
         return value.strftime(format)
     except Exception as e:
-        Log.error(u"Can not format {{value}} with {{format}}", value=value, format=format, cause=e)
+        Log.error(
+            "Can not format {{value}} with {{format}}",
+            value=value,
+            format=format,
+            cause=e,
+        )
 
 
 def join_path(*path):
     def scrub(i, p):
         p = p.replace(os.sep, "/")
-        if p in ('', '/'):
+        if p in ("", "/"):
             return "."
-        if p[-1] == '/':
+        if p[-1] == "/":
             p = p[:-1]
-        if i > 0 and p[0] == '/':
+        if i > 0 and p[0] == "/":
             p = p[1:]
         return p
 
     path = [p._filename if isinstance(p, File) else p for p in path]
-    abs_prefix = ''
+    abs_prefix = ""
     if path and path[0]:
-        if path[0][0] == '/':
-            abs_prefix = '/'
+        if path[0][0] == "/":
+            abs_prefix = "/"
             path[0] = path[0][1:]
-        elif os.sep == '\\' and path[0][1:].startswith(':/'):
+        elif os.sep == "\\" and path[0][1:].startswith(":/"):
             # If windows, then look for the "c:/" prefix
             abs_prefix = path[0][0:3]
             path[0] = path[0][3:]
@@ -557,7 +604,7 @@ def join_path(*path):
             pass
         elif s == "..":
             if simpler:
-                if simpler[-1] == '..':
+                if simpler[-1] == "..":
                     simpler.append(s)
                 else:
                     simpler.pop()
@@ -574,7 +621,7 @@ def join_path(*path):
         else:
             joined = "."
     else:
-        joined = abs_prefix + ('/'.join(simpler))
+        joined = abs_prefix + "/".join(simpler)
 
     return joined
 
@@ -582,6 +629,7 @@ def join_path(*path):
 def delete_daemon(file, caller_stack, please_stop):
     # WINDOWS WILL HANG ONTO A FILE FOR A BIT AFTER WE CLOSED IT
     from mo_threads import Till
+
     num_attempts = 0
     while not please_stop:
         try:
@@ -593,7 +641,9 @@ def delete_daemon(file, caller_stack, please_stop):
             e = Except.wrap(e)
             e.trace = e.trace[0:2] + caller_stack
             if num_attempts:
-                Log.warning(u"problem deleting file {{file}}", file=file.abspath, cause=e)
+                Log.warning(
+                    "problem deleting file {{file}}", file=file.abs_path, cause=e
+                )
             (Till(seconds=10) | please_stop).wait()
         num_attempts += 1
 
@@ -608,12 +658,3 @@ def add_suffix(filename, suffix):
     parts[i] = parts[i] + "." + text(suffix).strip(".")
     path[-1] = ".".join(parts)
     return File("/".join(path))
-
-
-def os_path(path):
-    """
-    :return: OS-specific path
-    """
-    if os.sep == "/":
-        return path
-    return str(path).lstrip("/")
